convert 1x/baseline_cancel_black_24dp.png -fill '#9AA7B0' -colorize 100 ../icons/1x/cancelled.png
convert 1x/baseline_check_circle_black_24dp.png  -fill '#62B543' -colorize 100 ../icons/1x/success.png
convert 1x/outline_warning_black_24dp.png -fill '#E05555' -colorize 100 ../icons/1x/error.png
convert 1x/baseline_change_circle_black_24dp.png -fill '#F4AF3D' -colorize 100 ../icons/1x/running.png
convert 1x/baseline_pending_black_24dp.png -fill '#40B6E0' -colorize 100 ../icons/1x/pending.png

convert 2x/baseline_cancel_black_24dp.png -fill '#9AA7B0' -colorize 100 ../icons/2x/cancelled.png
convert 2x/baseline_check_circle_black_24dp.png  -fill '#62B543' -colorize 100 ../icons/2x/success.png
convert 2x/outline_warning_black_24dp.png -fill '#E05555' -colorize 100 ../icons/2x/error.png
convert 2x/baseline_change_circle_black_24dp.png -fill '#F4AF3D' -colorize 100 ../icons/2x/running.png
convert 2x/baseline_pending_black_24dp.png-fill '#40B6E0' -colorize 100 ../icons/2x/pending.png
