main_font = 'Times New Roman'
#      This file is part of the PEP GUI detection pipeline batch running tool
#      Copyright (C) 2021 Yuval Boss yuval@uw.edu
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.

# main_font = 'Helvetica'
main_font = 'Arial'
class Fonts:
    description = (main_font, 10)
    description_bold = (main_font, 11, 'bold')

    title_large = (main_font, 18)
    title_medium = (main_font, 16)
    title_small = (main_font, 12)
    tab_text = (main_font, 11)