#      This file is part of the PEP GUI detection pipeline batch running tool
#      Copyright (C) 2021 Yuval Boss yuval@uw.edu
#
#      This program is free software: you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation, either version 3 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
__version__ = "N/A"
PLUGIN_PATH = os.path.normpath(os.path.dirname(os.path.abspath(__file__)))

version_txt_fp = os.path.join(os.path.dirname(__file__), 'VERSION.txt')
with open(version_txt_fp, 'r') as file:
    v_data = file.read().replace('\n', '')
if len(v_data.split('.')) == 3:
    __version__ = v_data
else:
    print(f'Invalid VERSION.cfg - could not parse {v_data}')